module CHIP_SASEBO_GII_CTRL
  (// FT2232D
   usb_d, usb_rdn, usb_wr, usb_rxfn, usb_txen,

   // cryptographic fpga
   zbus_clk, zbus_rstn, idata, odata, init, load, fetch, ack, EOM,

   // SelectMap configuration
   cfg_din, cfg_mosi, cfg_fcsb, cfg_cclk,
   cfg_progn, cfg_initn, cfg_csn, cfg_rdwrn, cfg_busy, cfg_done, cfg_done_alt,
   // Clock, reset, and LED.
   clkin, rstnin, led,
   debug);

   //------------------------------------------------
   // FT2232D
   inout [7:0]  usb_d;
   output       usb_rdn, usb_wr;
   input        usb_rxfn, usb_txen;

   // cryptographic fpga
   output zbus_clk;
   output zbus_rstn;
   output [15:0] idata;
   output load;
   output fetch;
   output init;
   input ack;
   input [15:0] odata;
   
   output [15:0] debug;
   reg debug_r0;

   output EOM;

   // Local bus SASEBO_R
   wire [15:0]  lbus_a, lbus_di; 
   wire [15:0]  lbus_do;
   wire         lbus_w, lbus_r;

   // Local bus II
   wire [7:0] lbus_wd;
   wire       lbus_we;
   wire       lbus_ful, lbus_aful;
   wire [7:0] lbus_rd;
   wire       lbus_re;
   wire       lbus_emp, lbus_aemp;
   wire       lbus_rdy;


   // Virtex-5 configuration
   input        cfg_din, cfg_mosi, cfg_fcsb, cfg_cclk;
   input        cfg_progn;
   input        cfg_csn, cfg_initn, cfg_rdwrn, cfg_busy, cfg_done, cfg_done_alt;
   
   // clock, led
   input         clkin, rstnin; // Clock and reset input
   output [7:0]  led;           // LED
   
   //------------------------------------------------
   // Internal clock
   wire         clk, rst;
   assign clk = clkin;
   assign rst = ~(rstnin & cfg_done); 

   // Local bus
   wire [7:0] 	wd0,               wd1;
   wire 	we0, ful0, aful0,  we1, ful1, aful1;
   wire [7:0] 	rd0,               rd1;
   wire         re0, emp0, aemp0,  re1, emp1, aemp1;

   // etc
   reg [23:0]   cnt;
  
   wire [7:0]   wd, rd;
   wire         we, ful, aful, re, emp, aemp;
   wire [11:0]  rcnt;
   
   wire [15:0] ctrl; //#
   wire [2:0] state; //#
   wire blk_rstn; //#
   wire blk_drdy; //#
   wire blk_dvld; //#
   wire [15:0] blk_trig; //#  
   wire [15:0] ram_datain; //#  
   //assign debug[7:0] = usb_d;
   //assign debug[8] = usb_rxfn;
   //assign debug[9] = usb_txen;
   //assign debug[10] = usb_rdn;
   //assign debug[11] = usb_wr;
   //assign debug[12] = clkdv;
   //assign debug[13] = load;
   //assign debug[14] = ack;
   //assign debug[15] = fetch;
   //assign debug[11:8] = odata[3:0];
   assign debug[3:0] = odata[3:0]; //#
   //assign debug[7:4] = ram_datain[3:0]; //#
   assign debug[7:4] = ctrl[3:0]; //#
   //assign debug[6:4] = state; //#
   //assign debug[7] = blk_rstn; //#
   assign debug[8] = ctrl[4]; //#
   assign debug[9] = fetch; //#
   assign debug[10] = load; //#
   //assign debug[11] = init; //#
   assign debug[11] = ack; //#
   assign debug[12] = blk_drdy; //#
   assign debug[13] = blk_trig[0]; //#
   assign debug[14] = blk_dvld; //#
   assign debug[15] = zbus_clk; //#

   //------------------------------------------------
   assign led[0] = ~cnt[23];
   assign led[2] = rst;
   //assign led[1] = ~debug_r0;
   //assign led[2] = ~(lbus_rdy & cfg_done & cfg_done_alt);
   assign led[3] = ~(cfg_initn | cfg_progn | cfg_csn | cfg_rdwrn | cfg_busy);
   assign led[4] = ~cfg_din;
   assign led[5] = ~cfg_mosi;
   assign led[6] = ~cfg_fcsb;
   reg led_r;
   assign led[1] = ~led_r;
   assign led[7] = ~cfg_cclk;
   always @(posedge clk or posedge rst) begin
      if (rst) led_r <= 0;
      else if (EOM) led_r <= 1;
      else led_r <= led_r;
   end



   always @(posedge clk or posedge rst) begin 
      if (rst) debug_r0 <= 0;
      else if (usb_rxfn) debug_r0 <= 1;
      else debug_r0 <= debug_r0;
   end


   always @(posedge clk or posedge rst) 
     if (rst) cnt <= 24'h0;
     else     cnt <= cnt + 24'h1;

   //------------------------------------------------   
   CTRL_FT2232D ctr_ft2232d
     (.usb_d(usb_d), .usb_rdn(usb_rdn), .usb_wr(usb_wr),
      .usb_rxfn(usb_rxfn), .usb_txen(usb_txen),
      .wd(wd0), .we(we0), .ful(ful0), .aful(aful0),
      .rd(rd0), .re(re0), .emp(emp0), .aemp(aemp0),
      .lbus_rdy(lbus_rdy & cfg_done & cfg_done_alt),
      .clk(clk), .rst(rst));

   //------------------------------------------------ lbus_wr
   ASYNCFIFO_8x15 afifo_wr
     (.wd(wd0), .we(we0), .ful(ful0), .aful(aful0), .wclk(clk), .wrst(rst),
      .rd(rd1), .re(re1), .emp(emp1), .aemp(aemp1), .rclk(clk), .rrst(rst));

   BUS_CONNECTOR bus_wr 
     (.wd(lbus_wd), .we(lbus_we), .ful(lbus_ful), .aful(lbus_aful),
      .rd(rd1), .re(re1), .emp(emp1), .aemp(aemp1),
      .clk(clk), .rst(rst));

   //------------------------------------------------ lbus_rd
   ASYNCFIFO_8x15 afifo_rd
     (.wd(wd1), .we(we1), .ful(ful1), .aful(aful1), .wclk(clk), .wrst(rst),
      .rd(rd0), .re(re0), .emp(emp0), .aemp(aemp0), .rclk(clk), .rrst(rst));

   BUS_CONNECTOR bus_rd
     (.wd(wd1), .we(we1), .ful(ful1), .aful(aful1),
      .rd(lbus_rd), .re(lbus_re), .emp(lbus_emp), .aemp(lbus_aemp),
      .clk(clk), .rst(rst));
	//synthesis attribute KEEP of lbus_aemp is true;
   
   
   //------------------------------------------------   
   SYNCFIFO_8x2047 fifo_w
     (.wd(lbus_wd), .we(lbus_we), .ful(lbus_ful), .aful(lbus_aful),
      .rd(rd), .re(re), .emp(emp), .aemp(aemp),
      .cnt(rcnt), .clk(clk), .rst(rst));

   SYNCFIFO_8x2047 fifo_r
     (.wd(wd), .we(we), .ful(ful), .aful(aful),
      .rd(lbus_rd), .re(lbus_re), .emp(lbus_emp), .aemp(lbus_aemp),
      .cnt(), .clk(clk), .rst(rst));

   CTRL_LBUS ctrl_lbus
     (.lbus_a(lbus_a), .lbus_di(lbus_di), .lbus_do(lbus_do),
      .lbus_wr(lbus_w), .lbus_rd(lbus_r),
      .wd(wd), .we(we), .ful(ful), .aful(aful),
      .rd(rd), .re(re), .emp(emp), .aemp(aemp), .rcnt(rcnt),
      .clk(clk), .rst(rst));

   //------------------------------------------------   
   ZBUS_CONNECTOR zbus_connector
   (.idata(idata), .odata(odata), .init(init), .load(load), .fetch(fetch), .ack(ack),
   .lbus_a(lbus_a), .lbus_di(lbus_di), .lbus_do(lbus_do), .lbus_wr(lbus_w), .lbus_rd(lbus_r),
   .lbus_clk(clk), .lbus_rstn(~rst),
   .zbus_clk(zbus_clk), .zbus_rstn(zbus_rstn),
   //#
   .ctrl(ctrl), .state(state), .blk_rstn(blk_rstn),
   .blk_drdy(blk_drdy),
   .blk_dvld(blk_dvld),
   .blk_trig(blk_trig),
   .EOM(EOM), .ram_datain(ram_datain)
);

   
   assign lbus_rdy = ~rst;
   
endmodule // CHIP_SASEBO_GII_CTRL

