/*-------------------------------------------------------------------------
 Local bus interface for AES_Comp on FPGA of SASEBO 
 *** WARNING *** 
 This circuit works only with AES_Comp.
 Compatibility for another cipher module may be provided in future release.
 
 File name   : lbus_if.v
 Version     : Version 1.0
 Created     : SEP/01/2008
 Last update : SEP/01/2008
 Desgined by : Toshihiro Katashita
 
 
 Copyright (C) 2008 AIST
 
 By using this code, you agree to the following terms and conditions.
 
 This code is copyrighted by AIST ("us").
 
 Permission is hereby granted to copy, reproduce, redistribute or
 otherwise use this code as long as: there is no monetary profit gained
 specifically from the use or reproduction of this code, it is not sold,
 rented, traded or otherwise marketed, and this copyright notice is
 included prominently in any copy made.
 
 We shall not be liable for any damages, including without limitation
 direct, indirect, incidental, special or consequential damages arising
 from the use of this code.
 
 When you publish any results arising from the use of this code, we will
 appreciate it if you can cite our webpage
 (http://www.rcis.aist.go.jp/special/SASEBO/).
 -------------------------------------------------------------------------*/ 


//================================================ LBUS_IF
module LBUS_IF
  (lbus_a, lbus_di, 
     check_vld,
    // lbus_do, 
    lbus_wr, 
   // lbus_rd, // Local bus
   //blk_kin, 
   //blk_din, blk_dout, 
   //blk_krdy, 
   blk_drdy, 
   //blk_kvld, 
   blk_dvld,
   //#blk_encdec,
   trig_wr,
   //blk_en, 
   blk_rstn,
   lbus_clk, lbus_rstn,                                  // Clock and reset
   ctrl,
   state,
   blk_trig
   );
   
   //------------------------------------------------
   // Local bus
   input [15:0]   lbus_a;  // Address
   input [15:0]   lbus_di; // Input data  (Controller -> Cryptographic module)
   input          lbus_wr; // Assert input data
   //input          lbus_rd; // Assert output data
   //output [15:0]  lbus_do; // Output data (Cryptographic module -> Controller)
   input          lbus_clk;
   input          lbus_rstn;

   input check_vld;
   // Block cipher
   //output [127:0] blk_kin;
   //output [511:0] blk_din;
   //input [255:0]  blk_dout;
   //output         blk_krdy, 
   output         blk_drdy;
   //input          blk_kvld, 
   input          blk_dvld;
//#   output         blk_encdec; 
   output           trig_wr;
   //blk_en;
   output         blk_rstn;
   //Kazuo
   //# output [3:0] ctrl;
   output [15:0] ctrl;

   input [2:0] state;

   // Clock and reset
   wire         clk, rst;

   //------------------------------------------------
   //reg [15:0]    lbus_do;

   //reg [127:0]   blk_kin,  
   //reg [511:0]   blk_din;
   //reg           blk_krdy; 
   wire          blk_drdy;
//#   reg           blk_encdec;
   //wire          blk_en = 1;
   reg           blk_rstn;
   
   reg [1:0]     wr;
   reg           trig_wr_r;
   assign trig_wr = trig_wr_r;
   wire          ctrl_wr;
   //# reg [3:0]     ctrl; //#
   reg [15:0]     ctrl; //#
   //reg [15:0]    blk_trig;                      //data count
   output reg [15:0]    blk_trig;                      //data count

//   reg           read_standby; //#
   
   assign rst = ~lbus_rstn;
   assign clk = lbus_clk;
   
   //------------------------------------------------
   always @(posedge clk or posedge rst)
     if (rst) wr <= 2'b00;
     else     wr <= {wr[0],lbus_wr};
   
   always @(posedge clk or posedge rst)
     if (rst)            trig_wr_r <= 0;
     else if (wr==2'b01) trig_wr_r <= 1;
     else                trig_wr_r <= 0;
   
   assign ctrl_wr = (trig_wr & (lbus_a==16'h1000));
   
   always @(posedge clk or posedge rst) 
     //# if (rst) ctrl <= 4'b0000; //#
     if (rst) ctrl <= 16'h0000; //#
     else begin
        if (blk_drdy)       ctrl[0] <= 1;
        else if (state == 3'b110) ctrl[0] <= 0; //# when the load finished
        else if (|blk_trig) ctrl[0] <= 1;
        else if (blk_dvld)  ctrl[0] <= 0;

        if (state == 3'b110) ctrl[1] <= 1; //# when the load finished
        else if ((state == 3'b111) || (state == 3'b010)) ctrl[1] <= 0; //# start loading
 
        ctrl[2] <= ~blk_rstn;

        //#
        //else ctrl[3] <= 0;
        if (blk_dvld) ctrl[3] <= 0;
        else if (ctrl_wr) ctrl[3] <= lbus_di[3];
        else ctrl[3] <= ctrl[3];

        if (ctrl_wr) ctrl[4] <= lbus_di[4];
        if (ctrl_wr) ctrl[5] <= lbus_di[5];
        else if (check_vld) ctrl[5] <= 0;

        if (ctrl_wr) ctrl[15:8] <= lbus_di[15:8];
     end
   
   always @(posedge clk or posedge rst) 
     if (rst)          blk_trig <= 16'h0;
     else if (ctrl_wr) blk_trig <= {lbus_di[0],15'h0};
     else              blk_trig <= {1'h0,blk_trig[15:1]};

   assign blk_drdy = blk_trig[0];

   //always @(posedge clk or posedge rst) 
   //  if (rst)          blk_krdy <= 0;
   //  else if (ctrl_wr) blk_krdy <= lbus_di[1];This unconnected signal will be trimmed during the optimization process
   //  else              blk_krdy <= 0;

   always @(posedge clk or posedge rst) 
     if (rst)          blk_rstn <= 1;
     else if (ctrl_wr) blk_rstn <= ~lbus_di[2];
     else              blk_rstn <= 1;
   
   //------------------------------------------------

//  always @(posedge clk or posedge rst) begin
//     if (rst) begin
//        //# blk_encdec <= 0;
//        read_standby <= 0;
//        //blk_kin <= 128'h0;
//        //blk_din <= 512'h0;
//     end
//     else if (trig_wr) begin
///#         if (lbus_a==16'h2000) blk_encdec <= lbus_di[0];
///#         if (lbus_a==16'h1000) read_standby <= lbus_di[3];
//     end
//end
         
endmodule // LBUS_IF
