module ZBUS_CONNECTOR(
   idata, odata, init, load, fetch, ack,
   lbus_a, lbus_di, lbus_do, lbus_wr, lbus_rd,
   lbus_clk, lbus_rstn,
   zbus_clk, zbus_rstn,
   ctrl, state, blk_rstn,
   blk_drdy,
   blk_dvld,
   blk_trig, EOM, ram_datain
);

/***************** I/O ******************/
input [15:0] odata;
input ack;

output [15:0] idata;
output fetch;
output init;
output load;
output zbus_clk;
output zbus_rstn;

input [15:0] lbus_a;
input [15:0] lbus_di;
input lbus_wr;
input lbus_rd;
input lbus_clk;
input lbus_rstn;

output [15:0] lbus_do;

//#
//# output [3:0] ctrl;
output [15:0] ctrl;
output [2:0] state;
output blk_rstn, blk_drdy, blk_dvld;
output [15:0] blk_trig;
output EOM;
output [15:0] ram_datain;

/***************** wire ******************/ 
//# wire blk_drdy, blk_dvld, blk_encdec;
wire blk_drdy, blk_dvld;
wire blk_rstn;

wire trig_wr;

wire latency_check;
assign latency_check = ctrl[5];

wire [15:0] cycle_count;
wire [15:0] cycle_count_all;

//# wire [3:0] ctrl; //#
wire [15:0] ctrl; //#
wire [15:0] datain_read_addr;
wire dataout_we, datain_re;
wire [15:0] ram_addr;

wire [15:0] ram_datain;
wire ram_we;
wire [15:0] ram_dataout;
wire ram_re;
//output ram_re;

wire check_vld;
reg [15:0]idata_r;
wire [2:0] state;

wire read_allow; //#
wire clk;
assign zbus_clk = ~clk;

/***************** instansiation *********/

LBUS_IF lbus_if(
   .lbus_a(lbus_a), .lbus_di(lbus_di), //.lbus_do(lbus_do),
   .lbus_wr(lbus_wr), 
   .lbus_clk(lbus_clk), .lbus_rstn(lbus_rstn),
   .blk_drdy(blk_drdy), .trig_wr(trig_wr),
   //# .blk_dvld(blk_dvld), .blk_encdec(blk_encdec), .blk_rstn(blk_rstn),
   .blk_dvld(blk_dvld), .blk_rstn(blk_rstn),
   .ctrl(ctrl), .state(state), 
   .blk_trig(blk_trig), .check_vld(check_vld)
);

ZBUS_IF zbus_if(
   .lbus_clk(lbus_clk),
   .blk_drdy(blk_drdy),
   //# .blk_dvld(blk_dvld), .blk_encdec(blk_encdec), .blk_rstn(blk_rstn),
   .blk_dvld(blk_dvld), .blk_rstn(blk_rstn),
   .init(init), .load(load),
   .fetch(fetch), .ack(ack), .clk(clk), .rst_n(zbus_rstn),
   .datain_read_addr(datain_read_addr),
   .dataout_we(dataout_we), .datain_re(datain_re),
   .state(state), .latency_check(latency_check), .cycle_count(cycle_count), .check_vld(check_vld), .cycle_count_all(cycle_count_all),
   .read_allow(read_allow), .data_size(ctrl[15:8]) //#
);

RAM RAM(.ram_addr(ram_addr), .ram_datain(ram_datain), .ram_we(ram_we), .ram_dataout(ram_dataout), .ram_re(ram_re), .clk(lbus_clk));

assign ram_addr = (dataout_we)? datain_read_addr :                       //zbus_connector -> ctrl_lbus  
   (datain_re)? datain_read_addr                                      //zbus_connecror -> hash_fpga 
   : lbus_a ; 
assign ram_datain = (~dataout_we)? lbus_di : odata; 
assign ram_we = (trig_wr && (lbus_a != 16'h1000)) | dataout_we;
//# assign lbus_do = (lbus_a == 16'h1000)? {12'h000, ctrl} : //#
assign lbus_do = (lbus_a == 16'h1000)? ctrl : //#
   (lbus_a == 16'hFFFC)? 16'h4522 :
   (lbus_a == 16'h2000)? 16'h0000 : 
   (lbus_a == 16'h3000)? cycle_count : 
   (lbus_a == 16'h3200)? cycle_count_all : ram_dataout;


assign idata = idata_r;
always @(posedge lbus_clk or negedge zbus_rstn) begin
   if (~zbus_rstn) idata_r <= 0;
   else if (state == 3'b011) idata_r <= ram_dataout;
   else if (ack) idata_r <= ram_dataout;
   else idata_r <= idata_r;
end

assign ram_re = ((lbus_a != 16'h1000) && lbus_rd) | datain_re | ram_we;

assign read_allow = ctrl[3]; //#

assign EOM = ctrl[4];

endmodule

